require_relative 'treasure_trove'
require_relative 'playable'
require 'csv'

module StudioGame
  class Player
    include Playable
  
    attr_accessor :name, :health
  
    def initialize(name, health=100)
      @name = name.capitalize
      @health = health
      @found_treasures = Hash.new(0)
    end
  
    def to_s
    "I'm #{@name} with health = #{@health}, points = #{points}, and score = #{score}."
    end
  
    def score
      @health + points
    end
  
    def <=>(other_player)
      other_player.score <=> score
    end
  
    def name_and_health
      "#{@name} (#{@health})"
    end
  
    def found_treasure(treasure)
      @found_treasures[treasure.name] += treasure.points
      puts "#{@name} found a #{treasure.name} worth #{treasure.points} points."
      puts "#{@name}'s treasures: #{@found_treasures}"
    end
  
    def points
      @found_treasures.values.reduce(0) { |sum, x| sum + x }
    end
  
    def each_found_treasure
      @found_treasures.each do |name, points|
        treasure = Treasure.new(name, points)
        yield treasure
      end
    end
  
    def self.from_array(player_data)
      name, health = player_data
      Player.new(name, Integer(health))
    end
  end
end

if __FILE__ == $0
  player = Player.new("moe")
  puts player.name
  puts player.health
  player.w00t
  puts player.health
  player.blam
  puts player.health
end
